import useSWR from "swr";
import { useEffect } from "react";
import { useRouter } from "next/router";
import HttpService from "@/services/HttpService";
import queryString from "query-string";

export const useModel = () => {
  //use router to redirect
  const router = useRouter();
  //use swr to fetch models data
  const {
    data: models,
    error,
    revalidate,
  } = useSWR("/models", async () =>
    HttpService.get("/models")
      .then((res) => {
        return res?.data?.data;
      })
      .catch((error) => {
        if (error.response.status != 409) throw error;
        router.push("/verify-email");
      })
  );

  const getModelDetails = async ({ ...props }) => {
    HttpService.get(`/model/${props.id}/details`)
      .then((response) => {
        if (response?.status === 200) {
          props.setModelDetails(response?.data?.data);
        }
      })
      .catch((error) => {
        console.log("[error]", error.response.data.errors);
      });
  };

  const fetchAllModels = async ({ ...props }) => {
    //get query params from url
    const { page, gender, name, keyword, height } = props;
    if (page) {
      await HttpService.get(`/models?page=${page}`)
        .then((response) => {
          if (response?.status === 200) {
            props.setAllModels(response?.data?.data);
            return response?.data?.data
          }
        })
        .catch((error) => {
          console.log("[error]", error.response.data.errors);
        });
    } else if ((page && keyword) || height || name || gender) {
      //query parameters object
      const urlParams = {
        keyword: keyword,
        gender: gender,
        name: name,
        height: height,
      };
      const searchQuery = queryString.stringify(urlParams);
      console.log("search query", searchQuery);
      if (searchQuery !== "") {
        await HttpService.get(`/models?page=${page}&${searchQuery}`)
          .then((response) => {
            if (response?.status === 200) {
              props.setAllModels(response?.data?.data);
              return response?.data?.data
            }
          })
          .catch((error) => {
            console.log("[error]", error.response.data.errors);
          });
      }
    } else if (keyword || height || name || gender) {
      //query parameters object
      const urlParams = {
        keyword: keyword,
        gender: gender,
        name: name,
        height: height,
      };
      const searchQuery = queryString.stringify(urlParams);
      if (searchQuery !== "") {
        await HttpService.get(`/models?${searchQuery}`)
          .then((response) => {
            if (response?.status === 200) {
              console.log("[data]", response?.data?.data)
              props.setAllModels(response?.data?.data);
              return response?.data?.data
            }
          })
          .catch((error) => {
            console.log("[error]", error.response.data.errors);
          });
      }
    } else {
      await HttpService.get(`/models`)
        .then((response) => {
          if (response?.status === 200) {
            props.setAllModels(response?.data?.data);
            return response?.data?.data
          }
        })
        .catch((error) => {
          console.log("[error]", error);
        });
    }
  };

  const postModelApplication = async ({
    setLoading,
    setErrors,
    setPostModelApplicationStatus,
    email,
    first_name,
    last_name,
    height,
    birth_date,
    nationality,
    question,
    instagram,
    tiktok,
    image1,
    image2,
    image3,
  }) => {
    //send model application  post request

    //define data
    const formData = new FormData();

    formData.append("email", email);
    formData.append("first_name", first_name);
    formData.append("last_name", last_name);
    formData.append("height", height);
    formData.append("birth_date", birth_date);
    formData.append("nationality", nationality);
    formData.append("question", question);
    formData.append("instagram", instagram);
    formData.append("tiktok", tiktok);
    formData.append("image1", image1);
    formData.append("image2", image2);
    formData.append("image3", image3);

    //console
    console.log("Form Data");
    for (let obj of formData) {
      console.log(obj);
    }

    HttpService.post("/model/application", formData, {
      Headers: {
        "Content-type": "multipart/form-data",
      },
    })
      .then((response) => {
        if (response?.status === 200) {
          //model application successfully
          if (response.data.data) {
            setPostModelApplicationStatus(true);
            setLoading(false);
          }
        }
      })
      .catch((error) => {
        console.log("[error]", error);
        if (error.response.status != 422) throw error;
        setErrors(Object.values(error.response.data.errors).flat());
      });
  };

  const addModel = async ({
    setLoading,
    setErrors,
    name,
    gender,
    height,
    bust,
    waist,
    hips,
    dress,
    shoes_size,
    hair,
    eyes,
    instagram_id_link,
    profile_photo,
  }) => {
    //change loading valure
    setLoading(true);
    //send model application  post request
    //define data
    const formData = new FormData();

    formData.append("name", name);
    formData.append("gender", gender);
    formData.append("height", height);
    formData.append("bust", bust);
    formData.append("waist", waist);
    formData.append("hips", hips);
    formData.append("dress", dress);
    formData.append("shoes_size", shoes_size);
    formData.append("hair", hair);
    formData.append("eyes", eyes);
    formData.append("instagram_id_link", instagram_id_link);
    formData.append("profile_photo", profile_photo);

    //console
    console.log("Form Data");
    for (let obj of formData) {
      console.log(obj);
    }

    HttpService.post("/admin/add/model", formData, {
      Headers: {
        "Content-type": "multipart/form-data",
      },
    })
      .then((response) => {
        if (response?.status === 200) {
          setLoading(false);
          //mode is
          router.push("/admin/model/list");
        }
      })
      .catch((error) => {
        setLoading(false);
        console.log("[error]", error);
        if (error.response.status != 422) throw error;
        setErrors(Object.values(error.response.data.errors).flat());
      });
  };

  const updateModel = async ({ ...props }) => {
        //change loading value
        props.setLoading(true);
        //extract model
        const {name, gender, height, bust, waist, hips, dress, shoes_size, hair, eyes, instagram_id_link, profile_photo, setAllModels, setIsEditModel}=props;
        //send model application  post request
        //define data
        const formData = new FormData();
    
        formData.append("name", name);
        formData.append("gender", gender);
        formData.append("height", height);
        formData.append("bust", bust);
        formData.append("waist", waist);
        formData.append("hips", hips);
        formData.append("dress", dress);
        formData.append("shoes_size", shoes_size);
        formData.append("hair", hair);
        formData.append("eyes", eyes);
        formData.append("instagram_id_link", instagram_id_link);
        formData.append("profile_photo", profile_photo);
    
        //console
        console.log("Form Data");
        for (let obj of formData) {
          console.log(obj);
        }
    
        HttpService.post(`/admin/model/${props.id}`, formData, {
          Headers: {
            "Content-type": "multipart/form-data",
          },
        })
          .then((response) => {
            if (response?.status === 200) {
              //change loading status
              props.setLoading(false);
              //update success
              props.setUpdateModelSuccessStatus(response.data.data)
              //update models data
              const updateModels = fetchAllModels({setAllModels});
              console.log("[models]", updateModels)
              //model dat
              setAllModels(updateModels);
              //set edit model success status
              setIsEditModel?props.setIsEditModel(false):null;
              //mode is
              router.push("/admin/model/list");
            }
          })
          .catch((error) => {
            props.setLoading(false);
            console.log("[error]", error);
            if (error.response.status != 422) throw error;
            props.setErrors(Object.values(error.response.data.errors).flat());
          });
    
  };

  const deleteModel = async ({ ...props }) => {
    props.setLoading(true);

    HttpService.delete(`/admin/model/${props.id}`, {
      headers: {
        "Content-Type": "application/json",
      },
    })
      .then((response) => {
        if (response?.status === 200) {
          props.setDeleteModelSuccessStatus(response.data.data);
          props.setLoading(false);
        }
      })
      .catch((error) => {
        props.setDeleteModelSuccessStatus(false);
        props.setLoading(false);
        alert(Object.values(error.response.data.errors).flat());
      });
  };

  useEffect(() => {}, [models, error]);

  return {
    models,
    fetchAllModels,
    addModel,
    postModelApplication,
    getModelDetails,
    deleteModel,
    updateModel
  };
};
